require('dotenv').config();
import * as db from './db';
import { getParams } from './functions';
import { getWorkspaceToken } from './slack/service';
import { IncomingMessage, ServerResponse } from 'http';

export const getSlackLink = (request: IncomingMessage, response: ServerResponse) => {
  const scope = 'chat:write,commands';
  const href = `https://slack.com/oauth/v2/authorize?client_id=${process.env.SLACK_CLIENT_ID}&scope=${scope}`;
  const htmlString = `<html>
    <a href="${href}">
      <img src="https://api.slack.com/img/sign_in_with_slack.png" />
    </a>
  </html>`;
  response.writeHead(200, { 'Content-Type': 'text/html' });
  response.end(htmlString);
};

export const authorize = async (request: IncomingMessage, _response: ServerResponse) => {
  let saved = false;
  let deleted = false;
  const params = getParams(request.url);
  const { code, error } = params;
  if (!error) {
    const data = await getWorkspaceToken(code);
    if (!data) {
      console.log('Could not get workspace token');
      return { message: 'Authentication failed. Could not get workspace token' };
    }
    const existingToken = await db.getTeamBotToken(data.team_id);
    if (existingToken) {
      if (existingToken === data.bot_access_token) {
        return { message: 'Workspace already authenticated!' };
      } else {
        console.log('Workspace has not matching data');
        deleted = await db.deleteOAuthResult(data.team_id);
        console.log(deleted ? 'OAuth was deleted' : 'OAuth was not deleted');
        saved = await db.saveOAuthResult(data);
      }
    } else {
      saved = await db.saveOAuthResult(data);
    }
  }
  console.log(saved ? 'OAuth saved' : 'OAuth not saved');
  return { message: saved ? 'Workspace authenticated!' : `Authentication failed. "${error || 'error saving to db'}"` };
};
