import Queue from 'bull';
import { QUEUE, JOB, CITIES } from './constants';

const REDIS_URL = process.env.REDIS_URL || 'redis://127.0.0.1:6379';

export const addFetchJobs = async () => {
  console.log('addFetchJobs');
  const promises = CITIES.map(async city => await mainQueue.add(JOB.FETCH_MEALS, city));
  return await Promise.all(promises);
};

export const mainQueue = new Queue(QUEUE.MAIN, REDIS_URL);

export async function initSuggestQueue() {
  console.log('Connecting to queue...');
  await mainQueue.isReady();
  const cleanJobs = await mainQueue.getJobs(['waiting', 'active', 'delayed', 'completed']); // Clean these jobs up when restarting server.
  console.log(`Removing ${cleanJobs.length} old jobs from queue`);
  await Promise.all(cleanJobs.map(job => job.remove()));
  const jobs = await mainQueue.getRepeatableJobs();
  console.log(`Removing ${jobs.length} existing repeatable jobs`);
  await Promise.all(jobs.map(job => mainQueue.removeRepeatableByKey(job.key)));

  console.log('Adding repeatable jobs to queue');
  await mainQueue.add(
    JOB.HEARTBEAT,
    {},
    {
      removeOnComplete: 2,
      repeat: { cron: '0 * * * *' },
    }
  );
  await mainQueue.add(
    JOB.SUGGEST_TO_EVERYONE,
    { name: 'Suggest to everyone' },
    {
      attempts: 3,
      backoff: {
        type: 'exponential',
        delay: 5000,
      },
      removeOnComplete: 20,
      repeat: { cron: '1 12 * * 1-5' },
    }
  );
  await mainQueue.add(
    JOB.ADD_FETCH_JOBS,
    {},
    {
      removeOnComplete: 20,
      repeat: { cron: '40,50,55 11 * * 1-5' },
    }
  );
  await mainQueue.add(
    JOB.ADD_FETCH_JOBS,
    {},
    {
      removeOnComplete: 20,
      repeat: { cron: '0,5,30,59 12 * * 1-5' },
    }
  );
}
