import { WebClient } from '@slack/web-api';
import { buildSuggestionBlocks, buildOptionsBlocks, Options, Option } from './suggestionsMessageBuilder';
import { DbMeal, DbDiner, User } from '../types';
import { CITIES, getCityName } from '../constants';

const slackWebClient = new WebClient();

export const sendCityOptions = async (user_name: string, token: string, city: string | null): Promise<boolean> => {
  const options: Options = {
    message: 'Vali linn',
    name: 'city',
    options: CITIES.map<Option>(city => ({
      description: getCityName(city),
      data: [city],
    })),
    selectText: city ? getCityName(city) : '...',
  };

  const blocks = buildOptionsBlocks('', options);
  try {
    const result = await slackWebClient.chat.postMessage({
      token: token,
      blocks,
      text: 'Vali linn',
      channel: `@${user_name}`,
      as_user: true,
    });
    return result.ok;
  } catch (e) {
    console.error(e);
    console.log(JSON.stringify(blocks));
    return false;
  }
};

export const sendSuggestionsOptions = async (
  user_name: string,
  token: string,
  suggest_favorites: boolean
): Promise<boolean> => {
  const options: Options = {
    message: 'Soovin pakkumisi saada',
    name: 'suggest_favorites',
    options: [
      { description: 'soovitajalt', data: ['0'] },
      { description: 'lemmikutest', data: ['1'] },
    ],
    selectText: suggest_favorites ? 'lemmikutest' : 'soovitajalt',
  };

  const blocks = buildOptionsBlocks('', options);
  try {
    const result = await slackWebClient.chat.postMessage({
      token: token,
      blocks,
      text: 'Kas soovid saada lõuna ajal pakkumisi oma lemmikutest või soovitajalt?',
      channel: `@${user_name}`,
      as_user: true,
    });
    return result.ok;
  } catch (e) {
    console.error(e);
    console.log(JSON.stringify(blocks));
    return false;
  }
};

export const sendSuggestions = async (
  suggestions: Array<DbMeal>,
  diners: Array<DbDiner>,
  user: User
): Promise<boolean> => {
  if (suggestions.length === 0 && diners.length === 0) {
    const result = await slackWebClient.chat.postMessage({
      token: user.token,
      text: 'Tänaseid päevapakkumisi pole veel laetud',
      channel: `@${user.name}`,
      as_user: true,
    });
    return result.ok;
  }

  const blocks = buildSuggestionBlocks(diners, suggestions);
  try {
    const result = await slackWebClient.chat.postMessage({
      token: user.token,
      blocks,
      text: 'Tulemus: ',
      channel: `@${user.name}`,
      as_user: true,
    });
    return result.ok;
  } catch (e) {
    console.error(e);
    console.log(JSON.stringify(blocks));
    return false;
  }
};

export const sendDiners = async (diners: Array<DbDiner>, meals: Array<DbMeal>, user: User): Promise<boolean> => {
  if (diners.length === 0) {
    const result = await slackWebClient.chat.postMessage({
      token: user.token,
      text: 'Sellise nimega söögikohta ei leidnud :(',
      channel: `@${user.name}`,
      as_user: true,
    });
    return result.ok;
  }

  const blocks = buildSuggestionBlocks(diners, meals);
  try {
    const result = await slackWebClient.chat.postMessage({
      token: user.token,
      text: 'Otsingu tulemused',
      blocks,
      channel: `@${user.name}`,
      as_user: true,
    });
    return result.ok;
  } catch (e) {
    console.error(e);
    console.log(JSON.stringify(blocks));
    return false;
  }
};
