import axios from 'axios';
import { stringify } from 'querystring';
import { getPostData } from '../functions';
import { OAuthResult } from '../types';
import { IncomingMessage } from 'http';

export const getWorkspaceToken = async (code: string) => {
  const url = 'https://slack.com/api/oauth.v2.access';
  const headers = {
    'Content-Type': 'application/x-www-form-urlencoded',
    Accept: 'application/json',
  };
  const requestBody = {
    client_id: process.env.SLACK_CLIENT_ID,
    client_secret: process.env.SLACK_CLIENT_SECRET,
    code,
  };

  const result = await axios.post(url, stringify(requestBody), { headers });
  console.log(JSON.stringify(result.data));
  if (!result.data.ok) return;
  const {
    data: {
      ok,
      //app_id,
      authed_user: { id: user_id },
      scope,
      //token_type,
      access_token,
      bot_user_id,
      team: { id: team_id, name: team_name },
    },
  } = result;
  const bot_access_token = access_token;
  const data: OAuthResult = {
    ok,
    scope,
    team_id,
    team_name,
    bot_user_id,
    bot_access_token,
    user_id,
    access_token: '',
  };
  return data;
};

export async function getSlackPostData(
  request: IncomingMessage,
  command: string
): Promise<{
  data: any;
  user_id: string;
  team_id: string;
  user_name: string;
  verified: boolean;
  text: string;
}> {
  const data = await getPostData(request);
  const verified = data.command === command && data.token === process.env.SLACK_VERIFICATION_TOKEN;
  return {
    data,
    user_id: data.user_id,
    team_id: data.team_id,
    user_name: data.user_name,
    text: data.text,
    verified,
  };
}
