import { DbDiner, DbMeal } from '../types';

export interface Options {
  message: string;
  selectText: string;
  name: string;
  options: Array<Option>;
}

export interface Option {
  description: string;
  data: Array<string>;
}

export const buildOptionsBlocks = (message: string, options: Options) => {
  return [
    ...(message
      ? [
          {
            type: 'section',
            text: {
              type: 'plain_text',
              emoji: true,
              text: message,
            },
          },
        ]
      : []),
    buildOptionsSelect(options),
    {
      type: 'section',
      text: {
        type: 'mrkdwn',
        text: 'Ei soovi valida',
      },
      accessory: {
        type: 'button',
        text: {
          type: 'plain_text',
          emoji: true,
          text: 'Sulge',
        },
        value: `dismiss`,
      },
    },
  ];
};

const buildOptionsSelect = (options: Options) => ({
  type: 'section',
  text: {
    type: 'mrkdwn',
    text: options.message,
  },
  accessory: {
    type: 'static_select',
    placeholder: {
      type: 'plain_text',
      text: options.selectText,
      emoji: true,
    },
    options: options.options.map(option => ({
      text: {
        type: 'plain_text',
        text: option.description,
        emoji: true,
      },
      value: `${options.name}|${option.data.join('|')}`,
    })),
  },
});

export const buildSuggestionBlocks = (diners: Array<DbDiner>, meals: Array<DbMeal>) => {
  const suggestions = meals.map(meal => meal.id).join(',');
  return [
    {
      type: 'section',
      text: {
        type: 'plain_text',
        emoji: true,
        text: 'Tänased valikud sulle:',
      },
    },
    ...buildDinersBlocks(diners, meals, suggestions),
    buildMealBlock(
      {
        id: undefined,
        description: 'Ükski soovitus ei sobi!',
        diner_id: 0,
        price: '',
      },
      suggestions
    ),
    {
      type: 'section',
      text: {
        type: 'mrkdwn',
        text: 'Ei soovi valida',
      },
      accessory: {
        type: 'button',
        text: {
          type: 'plain_text',
          emoji: true,
          text: 'Sulge',
        },
        value: `dismiss`,
      },
    },
  ];
};

const buildDinersBlocks = (diners: Array<DbDiner>, meals: Array<DbMeal>, suggestions: string) => {
  const blocks: Array<any> = [];
  diners.forEach(diner => {
    blocks.push({ type: 'divider' });
    blocks.push({
      type: 'section',
      text: {
        type: 'mrkdwn',
        text: diner.name_as_url
          ? `*<https://päevapakkumised.ee/${diner.city}/${diner.name_as_url}|${diner.name}>*\n${diner.address}\n${diner.district}`
          : diner.homepage.length > 8
          ? `*<${diner.homepage}|${diner.name}>*\n${diner.address}\n${diner.district}`
          : `*${diner.name}*\n${diner.address}\n${diner.district}`,
      },
      accessory: {
        type: 'button',
        text: {
          type: 'plain_text',
          emoji: true,
          text: `${diner.favorite ? 'Eemalda lemmikutest' : 'Lisa lemmikutesse'}`,
        },
        value: `${diner.favorite ? 'unfavorite' : 'favorite'}|${diner.id}`,
      },
    });
    meals.filter(meal => meal.diner_id === diner.id).forEach(meal => blocks.push(buildMealBlock(meal, suggestions)));
  });
  return blocks;
};

const buildMealBlock = (meal: DbMeal, suggestions: string) => ({
  type: 'section',
  text: {
    type: 'mrkdwn',
    text: `${meal.description} - ${meal.price}`,
  },
  accessory: {
    type: 'button',
    text: {
      type: 'plain_text',
      emoji: true,
      text: 'Vali',
    },
    value: `choose|${meal.id}|${suggestions}`,
  },
});
