export interface Auth {
  bot_access_token: string;
  user_id: string;
  team_id: string;
  user_name: string;
  suggest_favorites?: boolean;
}

export interface DbChoice {
  [index: string]: any;
  meal_id?: string;
  user_id: string;
  team_id: string;
  suggestions: string;
}

export interface DbUser {
  [index: string]: any;
  user_id: string;
  team_id: string;
  user_name: string;
  city: string | null;
  district: string | null;
}

export interface UserSettings {
  city: string | null;
  suggest_favorites: boolean;
}

export interface DbDiner {
  [index: string]: any;
  id: number;
  name: string;
  address: string;
  district: string;
  homepage: string;
  h_coordinate: string | null;
  l_coordinate: string | null;
  city: string;
  name_as_url: string;
  favorite?: boolean;
}

export interface DbMeal {
  [index: string]: any;
  id?: number;
  diner_id: number;
  price: string;
  description: string;
}

export interface DbClassifiedMeal {
  id: number;
  diner_id: number;
  price: string;
  description: string;
  classifier: string;
  category: string;
  lemmas: Array<string>;
  brackets: Array<string>;
  meat: Array<string>;
  ingredients: Array<string>;
  vegetarian: boolean | null;
  l_coordinate: number | null;
  h_coordinate: number | null;
}

export interface DbClassifiedChoice extends DbClassifiedMeal {
  user_id: string;
  team_id: string;
  suggestions: Array<number>;
  created_at: string;
}

export interface MealResponse {
  description: string;
  price: string;
}

export interface MealsResponse {
  meals: Array<MealResponse>;
  diner: {
    id: number;
    name: string;
    nameAsUrl: string;
    address: string;
    county: string;
    district: string;
    homepage: string;
    coordinates: { h: string; l: string };
    status: string;
    public: boolean;
    active: boolean;
  };
}

export interface User {
  token: string;
  name: string;
}

export interface Suggestions {
  user_id: string;
  team_id: string;
  meal_ids: Array<number>;
  debug?: any;
}

export interface RecommenderResponse {
  suggestions: Array<Suggestions>;
  errors: Array<any>;
}

export interface RecommenderRequest {
  users: Array<{
    user_id: string;
    team_id: string;
  }>;
  meals: DbClassifiedMeal[];
  choices: DbClassifiedChoice[];
  debug: boolean;
}

export interface OAuthResult {
  [key: string]: any;
  readonly ok: boolean;
  access_token: string;
  scope: string;
  user_id: string;
  team_id: string;
  team_name: string;
  bot_user_id: string;
  bot_access_token: string;
}

export interface ClassifierRequest {
  meals: Array<{
    id: number;
    description: string;
  }>;
}

export interface ResultMeal {
  brackets: Array<string>;
  category: string | null;
  classifier: string;
  description: string;
  id: number;
  ingredients: Array<string>;
  lemmas: Array<string>;
  meat: Array<string>;
  vegetarian: boolean | null;
  error?: boolean;
}

export interface ClassifierResponse {
  meals: Array<ResultMeal>;
}
