CREATE TABLE oauth (
	team_id varchar(255) NOT NULL,	
	access_token varchar(255) NOT NULL,
	scope varchar(255) NOT NULL,
	user_id varchar(255) NOT NULL,
	team_name varchar(255) NOT NULL,
	bot_user_id varchar(255) NOT NULL,
	bot_access_token varchar(255) NOT NULL,
	created_at DATETIME DEFAULT CURRENT_TIMESTAMP,
  PRIMARY KEY (team_id)
)
ENGINE=InnoDB
DEFAULT CHARSET=utf8
COLLATE=utf8_unicode_ci;

CREATE TABLE users (
	user_id varchar(255) NOT NULL,
	team_id varchar(255) NOT NULL,
	user_name varchar(255) NOT NULL,
	city varchar(255) DEFAULT NULL,
	suggest_favorites BOOLEAN DEFAULT 0 NOT NULL,
	district varchar(255) DEFAULT NULL,
	created_at DATETIME DEFAULT CURRENT_TIMESTAMP,
	deleted_at DATETIME DEFAULT NULL,
	suggested_at DATETIME DEFAULT CURRENT_TIMESTAMP,
  PRIMARY KEY (user_id, team_id) 
)
ENGINE=InnoDB
DEFAULT CHARSET=utf8
COLLATE=utf8_unicode_ci;

CREATE TABLE diners (
	id int NOT NULL,
	name varchar(255) NOT NULL,
	address varchar(255) NOT NULL,
	district varchar(127) NOT NULL,
	homepage varchar(127) NOT NULL,
	h_coordinate varchar(10),
	l_coordinate varchar(10),
	city varchar(127) NOT NULL,
	name_as_url varchar(127) NOT NULL,
  PRIMARY KEY (id)
)
ENGINE=InnoDB
DEFAULT CHARSET=utf8
COLLATE=utf8_unicode_ci;

CREATE TABLE meals (
	id int NOT NULL AUTO_INCREMENT,
	diner_id int NOT NULL,
	description varchar(511) NOT NULL,
	price varchar(15) NOT NULL,
	created_at DATETIME DEFAULT CURRENT_TIMESTAMP,
  PRIMARY KEY (id),
	FOREIGN KEY (diner_id) REFERENCES diners(id)
)
ENGINE=InnoDB
DEFAULT CHARSET=utf8
COLLATE=utf8_unicode_ci;

CREATE TABLE classifications (
	id int NOT NULL AUTO_INCREMENT,
	meal_id int NOT NULL UNIQUE,
	classifier varchar(255),
	category varchar(255),
	lemmas json not null,
	brackets json not null,
	meat json not null,
	ingredients json not null,
	vegetarian tinyint(1),
	created_at DATETIME DEFAULT CURRENT_TIMESTAMP,
  PRIMARY KEY (id),
	FOREIGN KEY (meal_id) REFERENCES meals(id)
)
ENGINE=InnoDB
DEFAULT CHARSET=utf8
COLLATE=utf8_unicode_ci;

CREATE TABLE choices (
	id int NOT NULL AUTO_INCREMENT,
	meal_id int,
	user_id varchar(255) NOT NULL,
	team_id varchar(255) NOT NULL,
	created_at DATETIME DEFAULT CURRENT_TIMESTAMP,
	suggestions json NOT NULL,
  PRIMARY KEY (id),
	FOREIGN KEY (meal_id) REFERENCES meals(id),
	FOREIGN KEY (user_id, team_id) REFERENCES users(user_id, team_id)
)
ENGINE=InnoDB
DEFAULT CHARSET=utf8
COLLATE=utf8_unicode_ci;

CREATE TABLE favorites (
	id int NOT NULL AUTO_INCREMENT,
	diner_id int,
	user_id varchar(255) NOT NULL,
	team_id varchar(255) NOT NULL,
	created_at DATETIME DEFAULT CURRENT_TIMESTAMP,
	deleted_at DATETIME DEFAULT NULL,
  PRIMARY KEY (id),
	FOREIGN KEY (diner_id) REFERENCES diners(id),
	FOREIGN KEY (user_id, team_id) REFERENCES users(user_id, team_id)
)
ENGINE=InnoDB
DEFAULT CHARSET=utf8
COLLATE=utf8_unicode_ci;
