import re
import estnltk

class Lemmatizer:
    pattern = re.compile(r'[\"\*\.\,\(\)\/\+\:\-\;\&\!\'\’\ø\\\?\=\d]')
    stop_words = [
        'ja', 'ning', 'või', 'v', 'and', 'g', 't', 'with', 'kui', 'ei', 'the',

        'tee', 'kohv', 'jäävesi', 'leib', 'maitsevesi', 'mahl', 'vein', 'klaas', 'piim',

        'punane', 'valge', 'roheline', 'suur', 'päikesekuivatatud', 'päiksekuivatatud',
        'värske', 'kodune', 'päev', 'lisand', 'valik', 'soe', 'külm', 'crispy', 'selge',
        'saatja', 'hommikukohv',

        'esmaspäev', 'teisipäev', 'kolmapäev', 'neljapäev', 'reede', 'laupäev', 'pühapäev',
        'ep', 'tp', 'kp', 'np', 'lp', 'pp',

        'avama|avatud', 'valmistama|valmistatud',
        'serveerima|serveeritud',
        'suletud|sulgema', 'küpsema|küpsenud', 'mixe|mixed',
        'maitsestama|maitsestatud',
        'saadaval', 'puusöeahi', 'tk', 'in', 'andma', 'olema', 'kergelt',
        'kogu', 'ahi', 'apollo', 'klubi', 'nädal', 'ainult', 'sheriff',
        'teenindaja', 'soovima', 'oma', 'kohvik', 'juurde', 'kõrvale',
        'kirju', 'ülek', 'prii', 'iga', 'talu', 'gv', 'pagaritoode', 'kuuluma',
        'dal', 'alg', 'panema', 'täidis', 'koha|koht', 'tempel', 'kuues', 'kuni', 'pikalt',
        'paneer', 'pakkumine', 'tooma', 'ahjutatud', 'pada', 'täna', 'külmkast', 'san',
        'kuu', 'kampaania', 'western', 'erinev', 'stiil',
        'liige', 'koos', 'kell', 'ost', 'hommikupakkumine', 'ürdine', 'mistahes', 'e-r',
        'panko', 'beans', 'moodi', 'terve', 'mutter', 'maitsev', 'pea', 'pakkuma', 'tartu',
        'paljas', 'polpo', 'eripakkumine', 'chef', 'poissmees', 'steakhouse', 'bartha', 'pipar', 'üleküpsetatu',
        'õrnalt', 'jõulupraad', 'specia', 'kerge', 'väike', 'lisa', 'värv', 'ranger', 'kauboi',
        'machine', 'lõunapakkumine', 'vürtsikashapu', 'startup', 'põhjal', 'traditsiooniline', 'äpi',
        'tükk', 'pall', 'day', 'ebamugavus', 'kaks', 'hug', 'leek', 'porx|porxi', 'madal', 'egg', 'rull',
        'pärast', 'rohke', 'alustama', 'eri', 'all', 'andre', 'võimalus', 'billy', 'tudeng', 'kõik', 'uus',
        'kid', 'kahjuks', 'eesti', 'talvine', 'mix', 'moter', 'mikroroheline', 'dets', 'nat', 'kõrval', 'big',
        'pink', 'hind', 'kehtima', 'saama', 'tasu', 'võik', 'vene', 'tn',

        'jaanuar', 'veebruar', 'märts', 'aprill', 'mai', 'juuni',
        'juuli', 'august', 'september', 'oktoober', 'november', 'detsember',
        'broneerima|broneeritud', 'kohtumine', 'aleksander', 'ahjusküpsetatud', 'märk',
        'teave', 'allergeen', 'kohta', 'klienditeenindaja', 'ask', 'info', 'about', 'allergens', 'order', 'desk',
        'kombo', 'vaba', 'vabandama', 'salgama', 'hea', 'isu', 'nb', 'nostalgia', 'käiguline', 'erimenüü',
        'püha', 'lõuna', 'väike|väikene', 'sina', 'mina', 'toiduvalik', 'la', 'carte', 'menüü', 'ootama',
        'reserveerima|reserveeritud', 'eest',
    ]

    def __should_discard(self, lemma):
        return len(lemma) < 3 or len(self.pattern.findall(lemma)) > 0 or (lemma in self.stop_words)

    def lemmatize(self, meal):
        if "http" in meal.description:
            raise Exception("Link in meal description")
        descriptionText = meal.description.strip().lower().replace('-', ' ')
        allLemmas = estnltk.Text(descriptionText).lemmas
        brackets = []

        lemmatizedText = " ".join(allLemmas)
        while '(' in lemmatizedText:
            bracketStart = lemmatizedText.find('(')
            bracketEnd = lemmatizedText[bracketStart+1:].find(')')
            bracketText = lemmatizedText[bracketStart+1: bracketEnd + bracketStart] if bracketEnd != -1 else lemmatizedText[bracketStart+1:]
            brackets += bracketText.strip().split(' ')
            endText = '' if bracketEnd == -1 else lemmatizedText[bracketEnd + bracketStart + 1:]
            lemmatizedText = lemmatizedText[:bracketStart].strip()+ ' ' + endText.strip()
            allLemmas = lemmatizedText.strip().split(' ')

        brackets = [lemma for lemma in brackets if not self.__should_discard(lemma)]
        filteredLemmas = [lemma for lemma in allLemmas if not self.__should_discard(lemma)]

        meal.lemmas = filteredLemmas
        meal.brackets = brackets
        return meal

class Classifier:
    name = 'DumbClassifier'
    
    def __countInString(self, lause, sonad):
        count = 0
        for sona in sonad:
            if sona in lause:
                count += 1
        return count

    def __classifyCategory(self, lause, meat, vegan):
        if self.__countInString(lause, [
            "idzumi", "tamagoyak", "sushi", "nigiri", "gunkan", "makk", "sashimi",
            "tempura",
        ]):
            return "SUSHI"
        
        if self.__countInString(lause, [
            "supp", "seljanka", "borš", "rassolnik", "hartšo", "minestrone", "guljašš", "soup",
            "boiullbai", "minestroon", "zuppa",
        ]):
            return "SUPP"
        

        if self.__countInString(lause, ["pitsa", "pizza"]):
            return "PITSA"

        if self.__countInString(lause, ["pasta", "lasanje", "lasagne", "nuudel", "nuudli", 
                                 "spaget", "ravioli", "gnocci", "pelmeen"]):
            return "PASTA"
        
        if self.__countInString(lause, ["puder"]):
            return "PUDER"
        
        if self.__countInString(lause, ["risotto", "orsoto", "orsotto", "risoto", 
                                 "riis", "paella", "kuskus", "kinoa",
                                "plohv", "pilaff", "lääts"]):
            return "RIIS"
        
        if self.__countInString(lause, [
            "magustoit", "kook", "jäätis", "kissell", "tarretis", "maius", "dessert",
            "tiramisu", "šokolaad", "puuviljasalat", "kohupiimavaht", "mannavaht", "tart",
            "smuuti", "tort", "vastlakukkel", "brülee", "brulee", "hüüve", "sidrunivaht",
            "vaniljekreem", "korvike", "moonirull", "puding", "toorjuustukreem", "cotta",
            "soolakaramell", "kaneelisai", "smuut", "biskviit", "kohupiimakreem",
            "kakaokreem", "kaneelirull", "pavlova", "mangokreem", "kamakreem", "koogi", "vanillikreem",
            "mangovaht", "kamavaht", "jogurtikreem", "kirsivaht", "mustikavaht", "muffin", "puuviljakreem",
            "profitrool", "kohvikreem", "sõstrakreem", "brownie", "tuuletasku", "küpsis", "kisell",
            "vaarikakreem", "kräsupea", "biskviid", "kardemonisai", "struudel", 'maasikakreem', "mustikakreem",
            "apelsinikreem", "virsikukreem", "marjavaht", "leivakreem", "moonisai",
            
        ]) and len(meat) == 0:
            return "MAGUSTOIT"
        
        if self.__countInString(lause, ["burger", "sub", "wrap", "vrap", "pirukas", "eineleib", "einesai", 
                                 "leivaeine", "taco", "enchil", "struudel"]):
            return "BURGER"
        
        if (self.__countInString(lause, [
            "salat"
        ]) and len(lause.split(' ')) <= 3) or self.__countInString(lause, [
            "caesar", "salatibuffet", "capres",
        ]):
            return "SALAT"
        
        if (self.__countInString(lause, [
            "paja", "tatar", "falafel", "carpaccio",
            "kartul", "šnitsel", "snitsel", "tamp", "kotlet", 
            "praad", "päevapakkumine", "roog",
        ]) or len(meat) > 0 or vegan):
            return "PRAAD"
        return None
    
    def __classifyMeat(self, lause):
        meat = []
        if "lihavaba" in lause:
            return meat
        
        if self.__countInString(lause, [
            "kana", "murgh", "duck", "chicken", "broiler", "kalkun", "pardi", "part", "paella", "vutt",
        ]):
            meat.append("LIND")
        
        if self.__countInString(lause, [
            "kala", "lõhe", "fish", "heeringas", "räim", "hõbeheik",
            "tursk", "lest", "ahven", "haug", "säga", "tuunikala", 
            "mereann", "krevet", "kalmaar", "karp", "aus", "krabi",
            "forell", "mereand", "koha", "saida", "latikas", "süsikas",
            "sardiin", "hõbeheig", "salmon", "tilaapia", "heigi",
        ]):
            meat.append("KALA")
        
        if self.__countInString(lause, [
            "frikadell", "seljanka", "rassolnik", "beef", "lasanje", "bolognes", "klops",
            "strooganov", "stroganoff", "pikkpoiss", "sea", "steik", "siga", "šašlõk", "maks", "kurzeme",
            "kebab", "lamba", "lammas", "veis", "peekon", "lihunik", "rostbiif", "steak",
            "verivorst", "sink", "guljašš", "talupoja", "hartšo", "azuu", "asuu", "pork",
            "suitsuvorst", "salaami", "speck", "salami", "pancetta", "singi", "biif", "eskalopp", "zuppa",
            "ribi", "raguu", "sardell", "teftel", "meat", "karbonaad", "grillvorst", "sašlõkk", "põsk",
            "plohv", "pilaff", 
            
        ]):
            meat.append("LOOM")
            
        if len(meat) == 0:
            if "liha" in lause:
                meat.append("LOOM")
        
        return meat
    
    
    def __classifyVegetarian(self, lause, meat):
        if len(meat) > 0:
            return False
        if self.__countInString(lause, ["falafe", "vegan", "päevavege", "vege", "taimne", "lihavaba"]):
            return True
        return None

    def __ingredients(self, lause, meat):
        i = []
        i += meat
        return i

    def classify(self, meal):
        lemmasString = " ".join(meal.lemmas)
        bracketsString = " ".join(meal.brackets)
        lemmasText = (lemmasString + ' ' + bracketsString).strip()

        meal.meat = self.__classifyMeat(lemmasText)
        meal.vegan = self.__classifyVegetarian(lemmasText, meal.meat)
        meal.category = self.__classifyCategory(lemmasText, meal.meat, meal.vegan)
        meal.ingredients = self.__ingredients(lemmasText, meal.meat)
        return meal
