from flask import Flask, request, jsonify
from collections import defaultdict
from Meal import Meal
from Classifier import Classifier, Lemmatizer
from Recommender import Recommender
import traceback

def exception_to_list(excp):
   stack = traceback.extract_tb(excp.__traceback__)  # add limit=?? 
   pretty = traceback.format_list(stack)
   pretty.reverse()
   pretty = map(lambda x: x.strip(), pretty)
   return list(pretty)

app = Flask(__name__)

@app.route('/', methods=['GET'])
def home():
    return jsonify({ "message": "ok" })

@app.route('/recommend', methods=['POST'])
def recommend():
    content = request.json
    users = content['users']
    meals = content['meals']
    choices = content['choices']
    debug = content['debug']
    
    errors = []
    suggestions = []
    recommender = Recommender(users, meals, choices, debug)
    for user in users:
        try:
            sugs = recommender.getSuggestions(user)
            suggestions.append(sugs)
        except Exception as e:
            print(e)
            errors.append({
                'user_id': user['user_id'], 
                'team_id': user['team_id'], 
                'message': str(e), 
                'type': str(type(e)),
                'trace': exception_to_list(e)
                })
    
    return jsonify({
        "suggestions": suggestions,
        "errors": errors
    })

@app.route('/classify', methods=['POST'])
def classify():
    content = request.json

    lemmatizer = Lemmatizer()
    classifier = Classifier()

    meals = []
    for mealJson in content['meals']:
        meal = Meal(mealJson['id'], mealJson['description'])
        try:
            meal.lemmatize(lemmatizer).classify(classifier)
        except Exception as e:
            meal.error = e
            print('\n'.join(exception_to_list(e)))
        
        mealDict = meal.toDict()
        meals.append(mealDict)
        if meal.error:
            print(mealDict)
    
    return jsonify({
        "meals": meals
    })

if __name__ == "__main__":
    print('__main__')
    app.run(debug=False, host='0.0.0.0')
